const express = require("express");
const User = require("./models/User");
const auth = require("./middleware/auth");
const config = require('./config');
const {nanoid} = require('nanoid');
const axios = require('axios');

const router = express.Router();

const createRouter = () => {
    router.get("/", async (req, res) => {
        res.send(await User.find());
    });
    router.post("/", async (req, res) => {
        try {
            const user = new User({
                username: req.body.username,
                email: req.body.email,
                password: req.body.password
            });
            user.generateToken();
            await user.save();
            res.send(user);
        } catch(e) {
            res.status(400).send(e);
        }
    });

    router.post("/sessions", async (req, res) => {
        const error = "Username or password are wrong";
        const user = await User.findOne({username: req.body.username});
        if (!user) {
            return res.status(400).send({error});
        }
        const isMatch = await user.checkPassword(req.body.password);
        if (!isMatch) {
            return res.status(400).send({error});
        }
        user.generateToken();

        await user.save({validateBeforeSave: false});
        res.send(user);
    });

    router.delete('/sessions', auth ,async (req, res) => {
        const user = req.user;
        const success = {message: 'Success'};

        user.token = '';
        await user.save({validateBeforeSave: false});

        return res.send(success);
    });

    router.post('/facebookLogin', async (req, res) => {
        const inputToken = req.body.accessToken;
        const accessToken = `${config.facebook.appId}|${config.facebook.secret}`;

        const tokenUrl = `https://graph.facebook.com/debug_token?input_token=${inputToken}&access_token=${accessToken}`;

        try{
            const response = await axios.get(tokenUrl);
            if(response.data.error) {
                return res.status(401).send({message: 'Facebook token error'});
            }
            if(response.data.data.user_id !== req.body.userID) {
                return res.status(401).send({message: 'Wrong User ID'});
            }
            let user = await User.findOne({facebookId: req.body.id});

            if(!user) {
                user = new User({
                    username: req.body.email,
                    email: req.body.email,
                    password: nanoid(),
                    facebookId: req.body.id
                })
            }

            user.generateToken();
            await user.save({validateBeforeSave: false});
            return res.send({message: 'Login or register successful', user});

        }catch (e) {
            return res.status(401).send({message: 'Facebook token error', e});
        }



    });

    return router;
};

module.exports = createRouter;